
import datetime as dt
import os
import shlex
import subprocess
import sys

import pytest

import ci_addons

from . import (push_dir)


def _run(cmd):
    print("Executing: %s" % cmd)
    subprocess.check_call(shlex.split(cmd))


def _expected_commit_yyyymmdd():
    commit_date = dt.datetime.now()
    return commit_date.strftime("%Y%m%d")


def _do_commit(release_tag=None):
    expected_commit_yyyymmdd = _expected_commit_yyyymmdd()
    msg = "Update %s" % expected_commit_yyyymmdd
    with open("README.md", "a") as content:
        content.write("* %s\n" % msg)
    # Commit changes
    _run("git add README.md")
    _run("git commit -m \"ENH: %s\"" % msg)
    # Create tag(s)
    if isinstance(release_tag, str):
        release_tag = [release_tag]
    if isinstance(release_tag, (list, tuple)):
        for tag in release_tag:
            _run("git tag %s -a -m \"ENH: %s\"" % (tag, tag))
    expected_sha = subprocess.check_output(
        ["git", "rev-parse", "--short", "HEAD"]).strip()
    if sys.version_info[0] >= 3:
        expected_sha = expected_sha.decode()
    return expected_sha, expected_commit_yyyymmdd


@pytest.fixture(scope="function")
def src_dir(tmpdir_factory):
    # Import addon to facilitate testing
    sys.path.insert(0, os.path.join(ci_addons.home(), 'anyci'))

    tmp_dir = tmpdir_factory.mktemp("source")
    with push_dir(tmp_dir):
        tmp_dir.ensure("dist", dir=1).join('dummy.txt').write("# dummy package")
        _run("git init")
        # Arbitrary user name and email
        _run("git config user.email "
             "scikit-ci-addons-bot@users.noreply.github.com")
        _run("git config user.name "
             "scikit-ci-addons-bot")
        return tmp_dir


@pytest.mark.parametrize("tags,extra_args", [
    ([], []),
    (["latest"], []),
    (["nightly"], ["--prerelease-tag", "nightly"]),
])
def test_create_prerelease(src_dir, mocker, tags, extra_args):

    pgr = __import__('publish_github_release')

    upload_prerelease = mocker.patch(
        'publish_github_release._upload_prerelease')
    upload_release = mocker.patch(
        'publish_github_release._upload_release')

    with push_dir(src_dir):
        _do_commit(release_tag=tags)
        pgr.main([
            "ci_addons", "anyci/publish_github_release",
            "--token", "123456",
            "--prerelease-packages", "dist/*",
            "--release-packages", "dist/*",
        ] + extra_args)
        assert upload_prerelease.call_count == 1
        assert upload_release.call_count == 0


@pytest.mark.parametrize("tags,extra_args", [
    (["1.0.0"], []),
    (["1.0.0", "latest"], []),
    (["latest"], ["--prerelease-tag", "nightly"]),
])
def test_create_release(src_dir, mocker, tags, extra_args):
    pgr = __import__('publish_github_release')

    upload_prerelease = mocker.patch(
        'publish_github_release._upload_prerelease')
    upload_release = mocker.patch(
        'publish_github_release._upload_release')

    with push_dir(src_dir):
        _do_commit(release_tag=tags)
        pgr.main([
            "ci_addons", "anyci/publish_github_release",
            "--token", "123456",
            "--prerelease-packages", "dist/*",
            "--release-packages", "dist/*",
        ] + extra_args)
        assert upload_prerelease.call_count == 0
        assert upload_release.call_count == 1
        if upload_release.call_count:
            release_tag = upload_release.call_args[0][0]
            # The first tag is expected to be the one
            assert release_tag == tags[0]


@pytest.mark.parametrize("tags,expected_prerelease,expected_release", [
    ([], 1, 0),
    (["1.0.0"], 0, 1),
])
def test_packages_selection_minilanguage(
        src_dir, mocker, tags, expected_prerelease, expected_release):
    pgr = __import__('publish_github_release')

    upload_prerelease = mocker.patch(
        'publish_github_release._upload_prerelease')
    upload_release = mocker.patch(
        'publish_github_release._upload_release')

    package_suffix = "<COMMIT_DATE>-<COMMIT_SHORT_SHA>*.txt"

    argv = [
        "ci_addons", "anyci/publish_github_release",
        "--token", "123456",
        "--prerelease-packages", "dist/prerelease*" + package_suffix,
        "--prerelease-packages-clear-pattern", "dist/clear*" + package_suffix,
        "--prerelease-packages-keep-pattern", "dist/keep*" + package_suffix,
        "--release-packages", "dist/release*" + package_suffix,
    ]

    with push_dir(src_dir):
        expected_sha, expected_commit_yyyymmdd = _do_commit(release_tag=tags)
        pgr.main(argv)
        assert upload_prerelease.call_count == expected_prerelease
        assert upload_release.call_count == expected_release

        expected_str = (expected_commit_yyyymmdd, expected_sha)

        if expected_prerelease:
            args = upload_prerelease.call_args[0][0]
            assert args.prerelease_packages == [
                "dist/prerelease*%s-%s*.txt" % expected_str]
            assert args.prerelease_packages_clear_pattern == \
                "dist/clear*%s-%s*.txt" % expected_str
            assert args.prerelease_packages_keep_pattern == \
                "dist/keep*%s-%s*.txt" % expected_str

        if expected_release == 1:
            release_tag = upload_release.call_args[0][0]
            assert release_tag == "1.0.0"
            args = upload_release.call_args[0][1]
            assert args.release_packages == [
                "dist/release*%s-%s*.txt" % expected_str]


def test_missing_token(src_dir, mocker, capsys, monkeypatch):
    monkeypatch.delenv("GITHUB_TOKEN")

    pgr = __import__('publish_github_release')

    upload_prerelease = mocker.patch(
        'publish_github_release._upload_prerelease')
    upload_release = mocker.patch(
        'publish_github_release._upload_release')

    with push_dir(src_dir):
        _do_commit()
        with pytest.raises(SystemExit) as excinfo:
            pgr.main([
                "ci_addons", "anyci/publish_github_release",
                "--prerelease-packages", "dist/*"
            ])
            assert upload_prerelease.call_count == 0
            assert upload_release.call_count == 0
            assert excinfo.value.code == 1

        out, _ = capsys.readouterr()
        assert "error: A token is expected." in out

        with pytest.raises(SystemExit) as excinfo:
            pgr.main([
                "ci_addons", "anyci/publish_github_release",
                "--exit-success-if-missing-token",
                "--prerelease-packages", "dist/*"
            ])
            assert upload_prerelease.call_count == 0
            assert upload_release.call_count == 0
            assert excinfo.value.code == 0

        out, _ = capsys.readouterr()
        assert "skipping: A token is expected." in out




"""
Usage::

    import install_cmake
    install_cmake.install()

"""

import errno
import os
import shutil
import sys
import zipfile

from subprocess import CalledProcessError, check_output

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen

DEFAULT_CMAKE_VERSION = "3.5.2"


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[appveyor:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def install(cmake_version=DEFAULT_CMAKE_VERSION):
    """Download and install CMake into ``C:\\cmake``.

    The function also make sure to prepend ``C:\\cmake\\bin``
    to the ``PATH``."""

    cmake_version_major = cmake_version.split(".")[0]
    cmake_version_minor = cmake_version.split(".")[1]
    cmake_directory = "C:\\cmake-{}".format(cmake_version)
    cmake_package = "cmake-{}-win32-x86.zip".format(cmake_version)

    _log("Looking for cmake", cmake_version, "in PATH")
    try:
        output = check_output(
            "cmake --version", shell=True).decode("utf-8")
        current_cmake_version = output.splitlines()[0]
        if cmake_version in current_cmake_version:
            _log("  ->", "found %s:" % current_cmake_version,
                 "skipping download: version matches expected one")
            return
        else:
            _log("  ->", "found %s:" % current_cmake_version,
                 "not the expected version")
    except (OSError, CalledProcessError):
        _log("  ->", "not found")
        pass

    _log("Downloading", cmake_package)
    if not os.path.exists(cmake_directory):
        remote_file = urlopen(
            "https://cmake.org/files/v{}.{}/{}".format(
                cmake_version_major, cmake_version_minor, cmake_package))

        with open("C:\\%s" % cmake_package, "wb") as local_file:
            shutil.copyfileobj(remote_file, local_file)
        _log("  ->", "done")

        _log("Unpacking", cmake_package, "into", cmake_directory)
        with zipfile.ZipFile("C:\\%s" % cmake_package) as local_zip:
            local_zip.extractall(cmake_directory)
        _log("  ->", "done")

        cmake_system_install_dir = "C:\\Program Files (x86)\\CMake"
        _log("Removing", cmake_system_install_dir)
        shutil.rmtree(cmake_system_install_dir)
        _log("  ->", "done")

        # C:\\cmake-3.6.2\\cmake-3.6.2-win32-x86
        cmake_package_no_ext = os.path.splitext(cmake_package)[0]
        inner_directory = cmake_directory + "\\" + cmake_package_no_ext

        _log("Moving", inner_directory, "to", cmake_system_install_dir)
        shutil.move(inner_directory, cmake_system_install_dir)
        shutil.rmtree(cmake_directory)
        _log("  ->", "done")

        # C:\\Program Files (x86)\\CMake\\bin\\cmake.exe
        cmake_exe = "%s\\bin\\cmake.exe" % cmake_system_install_dir
        _log("Checking if", cmake_exe, "exists")
        if os.path.exists(cmake_exe):
            _log("  ->", "found")
        else:
            # FileNotFoundError exception available only in python 3
            raise OSError(errno.ENOENT, "File not found", cmake_exe)

    else:
        _log("  ->", "skipping download: directory %s exists" % cmake_package)

    _log("Looking for cmake %s in PATH" % cmake_version)
    output = check_output(
        "cmake --version", shell=True).decode("utf-8")
    current_cmake_version = output.splitlines()[0]
    _log("  ->", "found %s" % current_cmake_version)


if __name__ == '__main__':
    install(sys.argv[1] if len(sys.argv) > 1 else DEFAULT_CMAKE_VERSION)




"""
Usage::

    import patch_vs2008
    patch_vs2008.apply_patch()

"""

import os
import shutil
import sys
import zipfile

from subprocess import check_call

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen

PATCH_URL = \
    "https://github.com/menpo/condaci/raw/master/vs2008_patch.zip"


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[appveyor:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def apply_patch():
    """Implement workaround for 64-bit Visual Studio 2008"""

    if os.environ["PYTHON_ARCH"] == "64":
        _log("Downloading 64-bit Visual Studio Fix")
        remote_zip = urlopen(PATCH_URL)
        with open("C:\\vs2008_patch.zip", "wb") as local_zip:
            shutil.copyfileobj(remote_zip, local_zip)

        _log("Unpacking 64-bit Visual Studio Fix")
        with zipfile.ZipFile("C:\\vs2008_patch.zip") as local_zip:
            local_zip.extractall("C:\\vs2008_patch")

        _log("Applying 64-bit Visual Studio Fix")
        check_call(
            [
                "cmd.exe",
                "/E:ON",
                "/V:ON",
                "/C",
                "C:\\vs2008_patch\\setup_x64.bat"
            ],
            cwd="C:\\vs2008_patch")


if __name__ == '__main__':
    apply_patch()




import os
from subprocess import check_output
import sys


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[%s] " % script_name + " ".join(args))
    sys.stdout.flush()


expected = sys.argv[1]
_log("expected:", expected)

output = check_output(
    "cmake --version", shell=True).decode("utf-8")
current = output.splitlines()[0].split()[2]
_log("current:", current)

assert current == expected




import errno
import os

from contextlib import contextmanager


def captured_lines(cap):
    """Given a ``capsys`` or ``capfd`` pytest fixture, return
     a tuple of the form ``(out_lines, error_lines)``.

    See http://doc.pytest.org/en/latest/capture.html
    """
    out, err = cap.readouterr()
    return (out.replace(os.linesep, "\n").split("\n"),
            err.replace(os.linesep, "\n").split("\n"))


def format_args_for_display(args):
    """Format a list of arguments appropriately for display. When formatting
    a command and its arguments, the user should be able to execute the
    command by copying and pasting the output directly into a shell.

    Currently, the only formatting is naively surrounding each argument with
    quotation marks.
    """
    return ' '.join("\"{}\"".format(arg) for arg in args)


def mkdir_p(path):
    """Ensure directory ``path`` exists. If needed, parent directories
    are created.

    Adapted from http://stackoverflow.com/a/600612/1539918
    """
    try:
        os.makedirs(path)
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:  # pragma: no cover
            raise


@contextmanager
def push_dir(directory=None, make_directory=False):
    """Context manager to change current directory.

    :param directory:
      Path to set as current working directory. If ``None``
      is passed, ``os.getcwd()`` is used instead.

    :param make_directory:
      If True, ``directory`` is created.
    """
    old_cwd = os.getcwd()
    if directory:
        if make_directory:
            mkdir_p(directory)
        os.chdir(str(directory))
    yield
    os.chdir(old_cwd)




"""
Usage::

    import apply_mingw_path_fix
    apply_mingw_path_fix.update_path()

"""

import os
import sys


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[appveyor:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def update_path():
    """Implement workaround for MinGW on Appveyor"""
    if os.environ.get("CMAKE_GENERATOR", "").lower().startswith("mingw"):
        _log("Applying MinGW PATH fix")

        mingw_bin = os.path.normpath(
            os.path.join("C:\\", "MinGW", "bin")).lower()

        os.environ["PATH"] = os.pathsep.join(
            one_path for one_path in
            os.environ["PATH"].split(os.pathsep)

            if (
                os.path.normpath(one_path).lower() == mingw_bin or
                not (
                    os.path.exists(os.path.join(one_path, "sh.exe")) or
                    os.path.exists(os.path.join(one_path, "sh.bat")) or
                    os.path.exists(os.path.join(one_path, "sh"))
                )
            )
        )


if __name__ == '__main__':
    update_path(sys.argv[0])




"""
Usage::

    import install_pyenv
    install_pyenv.install("3.4.5")

"""

import os
import sys
import tempfile
import textwrap

from subprocess import check_output


def _log_prefix():
    script_name = os.path.basename(__file__)
    return "[travis:%s] " % script_name


def _log(*args):
    print(_log_prefix() + " ".join(args))
    sys.stdout.flush()


def indent(text, prefix, predicate=None):
    """Adds 'prefix' to the beginning of selected lines in 'text'.
    If 'predicate' is provided, 'prefix' will only be added to the lines
    where 'predicate(line)' is True. If 'predicate' is not provided,
    it will default to adding 'prefix' to all non-empty lines that do not
    consist solely of whitespace characters.

    Copied from textwrap.py available in python 3 (cpython/cpython@a2d2bef)
    """
    if predicate is None:
        def predicate(line):
            return line.strip()

    def prefixed_lines():
        for line in text.splitlines(True):
            yield (prefix + line if predicate(line) else line)
    return ''.join(prefixed_lines())


def _execute_script(script):
    def _write(output_stream, txt):
        output_stream.write(bytearray("%s\n" % txt, "utf-8"))
    with tempfile.NamedTemporaryFile(delete=True) as script_file:
        _write(script_file, script)
        script_file.file.flush()
        # _log("Executing:", "bash", script_file.name)
        return check_output(
            ["bash", script_file.name]).decode("utf-8").strip()


def is_pyenv_installed(py_version):
    """Return True if ``py_version`` pyenv is installed.
    """
    script = textwrap.dedent(
        r"""
        #eval "$( pyenv init - )"
        (pyenv versions             \
          | sed -Ee "s/\(.+\)//"    \
          | tr -d "* "              \
          | grep "^{py_version}$")  \
        || echo ""
        """.format(py_version=py_version)
    )
    return _execute_script(script) == py_version


def pyenv_executable_path(py_version, executable="python"):
    return os.path.expanduser(
        "~/.pyenv/versions/%s/bin/%s" % (py_version, executable))


def pyenv_executable_exists(py_version, executable="python"):
    return os.path.exists(pyenv_executable_path(py_version, executable))


def install(py_version):
    """Update and install ``pyenv``."""

    _log("Looking for", pyenv_executable_path(py_version))
    python_found = pyenv_executable_exists(py_version)
    if python_found:
        _log("  ->", "found")
        return
    else:
        _log("  ->", "not found")

    cmd = "brew update"
    _log("Executing:", cmd)
    check_output(cmd, shell=True)
    _log("  -> done")

    cmd = "brew outdated pyenv || brew upgrade pyenv"
    _log("Executing:", cmd)
    check_output(cmd, shell=True)
    _log("  -> done")

    _log("Looking for pyenv", py_version)
    if is_pyenv_installed(py_version) and pyenv_executable_exists(py_version):
        _log("  ->", "found")
        return
    else:
        _log("  ->", "not found")

    _log("Installing pyenv", py_version)
    cmd = textwrap.dedent(
        """
        eval "$( pyenv init - )"
        pyenv install {py_version}
        """.format(py_version=py_version)
    ).strip()
    _log("Executing:")
    for line in indent(cmd, " " * 11).splitlines():
        _log(line)
    check_output(cmd, shell=True)
    _log("  -> done")

    _log("Looking for pyenv", py_version)
    if not is_pyenv_installed(py_version):
        exit(_log_prefix() +
             "  -> ERROR: Failed to install pyenv %s" % py_version)
    _log("  ->", "found")


if __name__ == '__main__':
    install(os.environ['PYTHON_VERSION'])




import os
import sys

current_version = list(sys.version_info[:3])
print("current: %s" % str(current_version))

expected_version = list(
    map(int, os.environ["EXPECTED_PYTHON_VERSION"].split(".")))
print("expected: %s" % str(expected_version))

assert current_version == expected_version



import os
import sys

from lxml import etree


def main(build_dir):

    tag_file = build_dir + "/Testing/TAG"
    if not os.path.exists(tag_file):
        raise RuntimeError("Missing tag file %s" % tag_file)

    with open(build_dir + "/Testing/TAG", 'r') as tag:
        dir_name = tag.readline().strip()

    test_xml = build_dir + "/Testing/" + dir_name + "/Test.xml"
    if not os.path.exists(test_xml):
        raise RuntimeError("Couldn't find %s" % test_xml)

    xsl_file = os.path.splitext(__file__)[0] + ".xsl"

    with open(xsl_file, 'r') as xsl:
        xml_doc = etree.parse(test_xml)
        xslt_root = etree.XML(xsl.read())
        transform = etree.XSLT(xslt_root)
        result_tree = transform(xml_doc)
        print(result_tree)

if __name__ == "__main__":
    main(sys.argv[1])




import anyci.docker
import ci_addons
import os
import py_compile
import pytest
import shutil
import subprocess
import sys

from lxml import etree

from . import captured_lines, format_args_for_display


def test_home():
    expected_home = os.path.abspath(os.path.dirname(__file__) + '/..')
    assert ci_addons.home() == expected_home


@pytest.mark.parametrize("addon, extension, exception, expected_prefix_path", [
    ('anyci/noop', '.py', None, ""),
    ('anyci/noop.py',  '.py', None, ""),
    ('noop', '.py', None, "anyci"),
    ('noop.py',  '.py', None, "anyci"),
    ('appveyor/patch_vs2008',  '.py', None, ""),
    (os.path.join(ci_addons.home(), 'appveyor/patch_vs2008'),  '.py', None, ""),
    ('travis/run-with-pyenv.sh',  '.sh', None, ""),
    ('install_cmake',  '.py', RuntimeError, ""),
    ('nonexistent',  '', RuntimeError, ""),
])
def test_path(addon, extension, exception, expected_prefix_path):
    expected_path = os.path.join(ci_addons.home(), expected_prefix_path, addon)
    if not addon.endswith(extension):
        expected_path += extension
    if exception is None:
        assert ci_addons.path(addon) == expected_path
    else:
        with pytest.raises(exception):
            ci_addons.path(addon)


def test_addons():
    noop_file_name = os.path.join("anyci", "noop") + ".py"
    noop_file_name_pyc = os.path.join("anyci", "noop") + ".pyc"
    noop_file_path = os.path.join(ci_addons.home(), noop_file_name)
    noop_file_path_pyc = os.path.join(ci_addons.home(), noop_file_name_pyc)

    py_compile.main([noop_file_path])

    if sys.version_info < (3,):
        assert os.path.exists(noop_file_path_pyc)

    assert noop_file_name in ci_addons.addons()
    assert noop_file_name_pyc not in ci_addons.addons()


@pytest.mark.parametrize("addon", ['anyci/noop', 'anyci/noop.py'])
def test_execute(addon, capfd):
    ci_addons.execute(addon, ['foo', 'bar'])
    output_lines, _ = captured_lines(capfd)
    noop_path = os.path.join(ci_addons.home(), 'anyci/noop.py')
    assert noop_path + ' foo bar' in output_lines


def test_install(tmpdir, capfd):
    noop = tmpdir.ensure('anyci/noop.py')

    ci_addons.install(str(tmpdir))
    output_lines, _ = captured_lines(capfd)

    for addon in ci_addons.addons():
        assert tmpdir.join(addon).exists()

    assert str(noop) + ' (skipped)' in output_lines
    assert str(tmpdir.join('appveyor', 'patch_vs2008.py')) in output_lines

    #
    # Check specifying --force overwrite add-ons already installed
    #
    ci_addons.install(str(tmpdir), force=True)
    output_lines, _ = captured_lines(capfd)

    assert str(noop) + ' (overwritten)' in output_lines

    #
    # Check that trying to overwrite original add-ons fails
    #
    with pytest.raises(RuntimeError):
        ci_addons.install(ci_addons.home())


def test_cli():

    root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    environment = dict(os.environ)
    environment['PYTHONPATH'] = root

    #
    # Running without argument should NOT fail
    #
    output = subprocess.check_output(
        "python -m ci_addons",
        shell=True,
        env=environment,
        stderr=subprocess.STDOUT,
        cwd=str(root)
    ).decode("utf-8")
    assert "usage:" in output

    #
    # Check that --list works
    #
    output = subprocess.check_output(
        "python -m ci_addons --list",
        shell=True,
        env=environment,
        stderr=subprocess.STDOUT,
        cwd=str(root)
    ).decode("utf-8")
    # Check that at least one add-on of each service is reported
    for addon in [
        "anyci/run.sh",
        "appveyor/rolling-build.ps1",
        "circle/install_cmake.py",
        "travis/run-with-pyenv.sh"
    ]:
        assert addon.replace('/', os.path.sep) in output


@pytest.mark.parametrize("filename, expected", [
    ("library/hello-world:latest", "library-hello-world-latest")
])
def test_addon_anyci_docker_get_valid_filename(filename, expected):
    assert anyci.docker.get_valid_filename(filename) == expected


def has_docker():
    """Return True if docker executable is found."""
    try:
        subprocess.check_output(["docker", "--version"])
        return True
    except (OSError, subprocess.CalledProcessError):
        return False


def test_addon_anyci_docker(tmpdir):

    test_image = "hello-world"
    test_image_filename = test_image + ".tar"
    test_image_id_filename = test_image + ".image_id"

    is_circleci = "CIRCLECI" in os.environ
    if is_circleci:
        assert has_docker(), "docker is expected when running tests on CircleCI"

    if not has_docker():
        pytest.skip("docker executable not found")

    root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    environment = dict(os.environ)
    environment['PYTHONPATH'] = root
    environment['HOME'] = str(tmpdir)

    def _display_cmd(cmd):
        print("\nExecuting: {}".format(format_args_for_display(cmd)))

    #
    # Delete image if any (useful when testing locally)
    #
    if not is_circleci:
        try:
            cmd = ["docker", "rmi", "-f", test_image]
            _display_cmd(cmd)
            subprocess.check_output(cmd)
        except subprocess.CalledProcessError:
            pass

    #
    # Check load-pull-save works with default cache directory
    #
    cmd = ["python", "-m",
           "ci_addons", "anyci/docker", "--",
           "load-pull-save", test_image, "--verbose"]
    _display_cmd(cmd)
    output = subprocess.check_output(
        cmd,
        env=environment,
        stderr=subprocess.STDOUT,
        cwd=str(root)
    ).decode("utf-8")
    assert "Status: Downloaded newer image for %s:latest" % test_image in output
    assert "cached image ID:" not in output
    assert tmpdir.join("docker", test_image_filename).exists()
    assert tmpdir.join("docker", test_image_id_filename).exists()

    #
    # Check load-pull-save works with custom cache directory
    #
    cache_dir = tmpdir.ensure("cache", dir=True)
    cmd_with_cache = cmd + ["--cache-dir", str(cache_dir)]
    _display_cmd(cmd_with_cache)
    output = subprocess.check_output(
        cmd_with_cache,
        env=environment,
        stderr=subprocess.STDOUT,
        cwd=str(root)
    ).decode("utf-8")
    assert "Status: Image is up to date for %s:latest" % test_image in output
    assert "cached image ID:" not in output
    assert tmpdir.join("cache", test_image_filename).exists()
    assert tmpdir.join("cache", test_image_id_filename).exists()

    #
    # Delete the image
    #

    cmd = ["docker", "rmi", "-f", test_image]
    _display_cmd(cmd)
    if not is_circleci:
        output = subprocess.check_output(cmd).decode("utf-8")
        assert "Untagged: %s@sha256" % test_image in output
        assert "Deleted: sha256:" in output
    else:
        print("  -> Skipping: "
              "Not supported on CircleCI because containers can NOT be "
              "removed in unprivileged LXC container")

    #
    # Check load-pull-save restores cached image
    #
    _display_cmd(cmd_with_cache)
    output = subprocess.check_output(
        cmd_with_cache,
        env=environment,
        stderr=subprocess.STDOUT,
        cwd=str(root)
    ).decode("utf-8")
    assert "cached image ID:" in output
    assert "Status: Image is up to date for %s:latest" % test_image in output
    assert "Skipped because pulled image did not change" in output
    assert tmpdir.join("cache", test_image_filename).exists()


def test_addon_anyci_ctest_junit_formatter(tmpdir):
    testing_dir = tmpdir.mkdir("Testing")
    tag = "20170426-0546"
    testing_dir.ensure("TAG").write(tag)

    # Input data and associated expected values were obtained by running Girder
    # tests using "ctest -D ExperimentalTest" and by copying the "Test.xml"
    # found in the Testing subdirectory.
    #
    # Note: If new data are generated, the expected values will have to be
    #       tweaked. Additionally, the "base64" section should be replaced by
    #       a shorter text.

    test_data_dir = os.path.join(os.path.dirname(__file__), "data")
    test_xml = os.path.join(test_data_dir, "ctest_junit_formatter_input.xml")
    assert os.path.exists(test_xml)

    shutil.copyfile(test_xml, str(testing_dir.mkdir(tag).join("Test.xml")))

    cmd = ["python", "-m",
           "ci_addons", "anyci/ctest_junit_formatter", str(tmpdir)]

    root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    environment = dict(os.environ)
    environment['PYTHONPATH'] = root
    environment['HOME'] = str(tmpdir)

    output = subprocess.check_output(
        cmd,
        env=environment,
        stderr=subprocess.STDOUT,
        cwd=root
    ).decode("utf-8", "ignore")

    expected_properties = {
        "BuildName": "Linux-",
        "BuildStamp": "20170426-0546-Experimental",
        "Name": "CerroTorre",
        "Generator": "ctest-3.4.2",
        "CompilerName": "",
        "OSName": "Linux",
        "Hostname": "CerroTorre",
        "OSRelease": "4.2.0-42-generic",
        "OSVersion": "#49-Ubuntu SMP Tue Jun 28 21:26:26 UTC 2016",
        "OSPlatform": "x86_64",
        "Is64Bits": "1",
        "VendorString": "GenuineIntel",
        "VendorID": "Intel Corporation",
        "FamilyID": "6",
        "ModelID": "94",
        "ProcessorCacheSize": "8192",
        "NumberOfLogicalCPU": "8",
        "NumberOfPhysicalCPU": "1",
        "TotalVirtualMemory": "63336",
        "TotalPhysicalMemory": "62267",
        "LogicalProcessorsPerPhysical": "8",
        "ProcessorClockFrequency": "3489.74"
    }

    output_doc = etree.XML(output)

    assert output_doc.tag == "testsuite"

    system_out = output_doc.findall("system-out")
    assert len(system_out) == 1

    properties = output_doc.findall("./properties/property")
    assert len(properties) == len(expected_properties)
    assert {elem.attrib["name"]: elem.attrib["value"]
            for elem in properties} == expected_properties

    testcases = output_doc.findall(
        "./testcase[@name][@classname='TestSuite'][@time]")
    assert len(testcases) == 193

    errors = output_doc.findall("./testcase/error[@message]")
    assert len(errors) == 37

    assert "testGroupAccess (tests.cases.group_test.GroupTestCase) ... FAIL" \
           in errors[0].text



"""
This add-on allows to automatically create GitHub releases or prereleases
and upload associated packages. It respectively provides a "release" and
a "prerelease" sub-command.
"""

import argparse
import datetime as dt
import errno
import glob
import os
import platform
import sys
import subprocess
import textwrap

from github_release import (
    gh_asset_delete,
    gh_asset_upload,
    get_refs,
    get_release_info,
    gh_release_create,
    gh_release_edit
)


#
# Git
#

GITS = ["git"]
if sys.platform == "win32":
    GITS = ["git.cmd", "git.exe"]


# Copied from https://github.com/warner/python-versioneer/blob/master/src/subprocess_helper.py  # noqa: E501
def run_command(commands, args, cwd=None, verbose=False, hide_stderr=False,
                env=None):
    """Call the given command(s)."""
    assert isinstance(commands, list)
    for c in commands:
        dispcmd = str([c] + args)
        try:
            # remember shell=False, so use git.cmd on windows, not just git
            p = subprocess.Popen([c] + args, cwd=cwd, env=env,
                                 stdout=subprocess.PIPE,
                                 stderr=(subprocess.PIPE if hide_stderr
                                         else None))
            break
        except EnvironmentError:
            e = sys.exc_info()[1]
            if e.errno == errno.ENOENT:
                continue
            if verbose:
                print("unable to run %s" % dispcmd)
                print(e)
            return None, None
    else:
        if verbose:
            print("unable to find command, tried %s" % (commands,))
        return None, None
    stdout = p.communicate()[0].strip()
    if sys.version_info[0] >= 3:
        stdout = stdout.decode()
    if p.returncode != 0:
        if verbose:
            print("unable to run %s (error)" % dispcmd)
            print("stdout was %s" % stdout)
        return None, p.returncode
    return stdout, p.returncode


def get_tags(ref="HEAD"):
    """If any, return all tags associated with `ref`.

    Note that since git (<=2.5.0) does *NOT* provide a direct way to get
    these, we first get the sha of the HEAD tag (if any), then group the
    the tag by SHA and return the corresponding list.
    """
    output, _ = run_command(
        GITS, ["describe", "--tags", "--exact-match", str(ref)],
        hide_stderr=True)
    if output is None:
        return []
    # Get tag's commit
    tag_sha, _ = run_command(GITS, ["rev-list", "-n",  "1", output])
    # List all tags and associated SHAs
    tags, _ = run_command(GITS, ["tag", "--list"])
    # map of sha -> tags
    all_tags = {}
    for tag in tags.splitlines():
        sha, _ = run_command(
            GITS, ["rev-list", "-n",  "1", "refs/tags/%s" % tag])
        if sha not in all_tags:
            all_tags[sha] = [tag]
        else:
            all_tags[sha].append(tag)
    return all_tags[tag_sha]


def get_current_date():
    now = dt.datetime.utcnow()
    return now.strftime("%Y-%m-%d %H:%m UTC")


def get_commit_date(ref="HEAD"):
    output, _ = run_command(
        GITS, ["log", "-1", "--format=%cd", "--date=local", str(ref)])
    return dt.datetime.strptime(
        output, "%a %b %d %H:%M:%S %Y").strftime("%Y%m%d")


def get_commit_short_sha(ref="HEAD"):
    output, _ = run_command(
        GITS, ["rev-parse", "--short=7", str(ref)])
    return output


#
# Python
#

def python_wheel_platform():
    this_platform = platform.system().lower()
    return {
        "linux": "manylinux1", "darwin": "macosx", "windows": "win"
    }.get(this_platform, this_platform)


#
# Mini-language for package selection
#

def _substitute_package_selection_strings(package, what):
    tokens = {
        '<PYTHON_WHEEL_PLATFORM>': python_wheel_platform,
        '<COMMIT_DATE>': get_commit_date,
        '<COMMIT_SHORT_SHA>': get_commit_short_sha
    }
    if any([token in package for token in tokens]):
        print("Updating %s [%s]" % (what, package))
    for token, replace_func in tokens.items():
        if token in package:
            updated_value = replace_func()
            print("  %s -> %s" % (token, updated_value))
            package = package.replace(token, updated_value)
    return package


def _update_package_list(input_packages, what):
    if input_packages is None:
        return input_packages
    packages = []
    if isinstance(input_packages, str):
        return _substitute_package_selection_strings(input_packages, what)
    for package in input_packages:
        packages.append(
            _substitute_package_selection_strings(package, what))
    return packages


#
# Entry point
#

def configure_parser(parser):
    parser.add_argument(
        "repo_name", type=str, metavar="ORG/PROJECT",
        help="Name of the repository"
    )
    # "release" arguments
    release_group = parser.add_argument_group('release')
    release_group.add_argument(
        "--release-packages", metavar="PATTERN", type=str, nargs="*",
        help="Path(s) and/or wildcard expression(s)"
    )
    # "prerelease" arguments
    prerelease_group = parser.add_argument_group('prerelease')
    prerelease_group.add_argument(
        "--prerelease-packages", metavar="PATTERN", type=str, nargs="*",
        help="Path(s) and/or globbing pattern(s)"
    )
    prerelease_group.add_argument(
        "--prerelease-packages-clear-pattern",
        type=str, metavar="PATTERN",
        help="Globbing pattern selecting the set of packages to remove"
    )
    prerelease_group.add_argument(
        "--prerelease-packages-keep-pattern",
        type=str, metavar="PATTERN",
        help="Globbing pattern identifying the subset of packages to keep"
    )
    prerelease_group.add_argument(
        "--prerelease-tag", type=str,
        help="Name of the tag to associate with the pre-release. "
             "If needed, tag is created or updated (default: latest)"
    )
    prerelease_group.add_argument(
        "--prerelease-name", type=str,
        help="Name of the pre-release "
             "(default: 'TagName (updated on YYYY-MM-DD HH:MM UTC)'. "
             "Example: 'Latest (updated on 2017-02-12 14:42 UTC)')"
    )
    prerelease_group.add_argument(
        "--prerelease-sha", type=str,
        help="Commit or branch name to associate with the pre-release "
             "(default: master)"
    )
    # Common arguments
    parser.add_argument(
        "--token", type=str, metavar="GITHUB_TOKEN",
        help="If not specified, expects GITHUB_TOKEN env. variable"
    )
    parser.add_argument(
        "--exit-success-if-missing-token", action="store_true",
        help="Gracefully exit if no GITHUB_TOKEN env. variable is found"
    )
    parser.add_argument(
        "--re-upload", action="store_true",
        help="If packages with same exist, delete and re-upload them"
    )
    parser.add_argument(
        "--display-python-wheel-platform", action="store_true",
        help="Display current python platform (manylinux1, macosx, or win)"
             " used for Python wheels."
    )
    parser.add_argument(
        "--dry-run", action="store_true",
        help="don't create release, upload or erase packages "
             "but act like it was done"
    )
    parser.set_defaults(
        token=os.environ.get("GITHUB_TOKEN", None),
        prerelease_packages_clear_pattern=None,
        prerelease_packages_keep_pattern=None,
        prerelease_name=None,
        prerelease_sha="master",
        prerelease_tag="latest"
    )


def _delete_matching_packages(repo_name, tag, packages):
    release = get_release_info(repo_name, tag)
    asset_names = [asset['name'] for asset in release['assets']]
    for package in packages:
        for path in glob.glob(package):
            local_asset_name = os.path.basename(path)
            # XXX Asset uploaded on GitHub always have "plus" sign found
            # in their name replaced by "dot".
            local_asset_name = local_asset_name.replace("+", ".")
            if local_asset_name in asset_names:
                gh_asset_delete(repo_name, tag, local_asset_name)


def _upload_release(release_tag, args):
    assert release_tag is not None
    # Create release
    gh_release_create(
        args.repo_name,
        release_tag,
        publish=True, prerelease=False
    )
    # Remove existing assets matching selected ones
    if args.re_upload:
        _delete_matching_packages(
            args.repo_name, release_tag, args.release_packages)
    # Upload packages
    gh_asset_upload(
        args.repo_name, release_tag, args.release_packages, args.dry_run)
    return True


def _cancel_additional_appveyor_builds(prerelease_tag):
    """
    This function is a workaround for
    https://github.com/scikit-build/scikit-ci-addons/issues/45
    """
    appveyor = os.environ.get("APPVEYOR", "").lower()
    if appveyor != "true":
        return
    print("")
    script_name = "cancel-queued-build.ps1"
    script = os.path.join(os.path.dirname(__file__), "../appveyor", script_name)
    if not os.path.exists(script):
        print("skipping AppVeyor job cancellation: "
              "script %s not found" % script)
        return

    tag_pattern = "^%s(-tmp)?$" % prerelease_tag

    subprocess.check_call([
        "powershell", "-ExecutionPolicy", "Unrestricted",
        "-file", script, "-tag_pattern", tag_pattern
    ])


def _upload_prerelease(args):
    # Set default prerelease name
    prerelease_name = args.prerelease_name
    if prerelease_name is None:
        prerelease_name = "%s (updated on %s)" % (
            args.prerelease_tag.title(), get_current_date()
        )
    # Create release
    gh_release_create(
        args.repo_name,
        args.prerelease_tag,
        name=prerelease_name,
        publish=True, prerelease=True
    )
    # Remove existing assets matching selected ones
    if args.re_upload:
        _delete_matching_packages(
            args.repo_name, args.prerelease_tag, args.prerelease_packages)
    # Upload packages
    gh_asset_upload(
        args.repo_name, args.prerelease_tag, args.prerelease_packages,
        args.dry_run
    )
    # Remove obsolete assets
    if args.prerelease_packages_clear_pattern is not None:
        gh_asset_delete(
            args.repo_name,
            args.prerelease_tag,
            args.prerelease_packages_clear_pattern,
            keep_pattern=args.prerelease_packages_keep_pattern,
            dry_run=args.dry_run
        )
    # If needed, update target commit
    sha = args.prerelease_sha
    if sha is not None:
        # If a branch name is specified, get associated commit
        refs = get_refs(args.repo_name, pattern="refs/heads/%s" % sha)
        if refs:
            assert len(refs) == 1
            branch = sha
            sha = refs[0]["object"]["sha"]
            print("resolved '%s' to '%s'" % (branch, sha))
        gh_release_edit(
            args.repo_name,
            args.prerelease_tag,
            target_commitish=sha
        )
    # If needed, update name associated with the release
    gh_release_edit(
        args.repo_name,
        args.prerelease_tag,
        name=prerelease_name
    )

    _cancel_additional_appveyor_builds(args.prerelease_tag)

    return True


def main(argv=None):
    parser = argparse.ArgumentParser(description=__doc__)
    configure_parser(parser)
    args = parser.parse_args(argv[1:] if argv else None)

    if args.display_python_wheel_platform:
        print(python_wheel_platform())
        exit(0)

    # Check required parameters
    upload_release = (len(args.release_packages) > 0
                      if args.release_packages else False)
    upload_prerelease = (len(args.prerelease_packages) > 0
                         if args.prerelease_packages else False)
    if not upload_release and not upload_prerelease:
        parser.print_usage()
        print("error: at least --release-packages or --prerelease-packages "
              "argument is required")
        sys.exit(1)
    dual = upload_release and upload_prerelease

    if not args.token:
        print("-" * 80)
        print(textwrap.dedent(
            r"""
            %s: A token is expected.

            Specify the --token parameter or set GITHUB_TOKEN environment variable.
            See https://help.github.com/articles/creating-an-access-token-for-command-line-use/
            """ % ("skipping" if args.exit_success_if_missing_token else "error")  # noqa: E501
        ).lstrip())
        print("-" * 80)
        parser.print_usage()
        exit(0 if args.exit_success_if_missing_token else 1)
    os.environ["GITHUB_TOKEN"] = args.token

    # Update package arguments
    args.release_packages = _update_package_list(
        args.release_packages, "release package")
    args.prerelease_packages = _update_package_list(
        args.prerelease_packages, "prerelease package")
    args.prerelease_packages_clear_pattern = _update_package_list(
        args.prerelease_packages_clear_pattern,
        "prerelease package clear pattern")
    args.prerelease_packages_keep_pattern = _update_package_list(
        args.prerelease_packages_keep_pattern,
        "prerelease package keep pattern")

    msg = "Checking if HEAD is a release tag"
    print(msg)

    head_tags = get_tags()
    head_tags = list(filter(lambda tag: tag != args.prerelease_tag, head_tags))

    is_release = False
    release_tag = None

    if len(head_tags) > 0:
        is_release = True
        # If there are more than one tag (different from args.prerelease_tag)
        # associated with the HEAD, we use the first one.
        release_tag = head_tags[0]

    # Upload
    uploaded = False
    if upload_release:
        if is_release:
            print("%s - yes "
                  "(found %s: creating release)\n" % (msg, release_tag))
            uploaded = _upload_release(release_tag, args)
        elif not dual:
            print("%s - no (skipping release upload)\n" % msg)

    if not uploaded and upload_prerelease:
        if not is_release:
            print("%s - no (creating prerelease)\n" % msg)
            _upload_prerelease(args)
        elif not dual:
            print("%s - yes (found %s: "
                  "skipping prerelease upload)\n" % (msg, release_tag))


if __name__ == "__main__":
    main()



import os
import struct

current_arch = struct.calcsize("P") * 8
print("current_arch: %s-bit" % current_arch)

expected_arch = int(os.environ["EXPECTED_PYTHON_ARCH"])
print("expected_arch: %s-bit" % expected_arch)

assert current_arch == expected_arch



"""
Add-on facilitating docker use on CI services.

It allows to load an image from local cache, pull and save back using
a convenience one-liner.

Usage::

    docker.py load-pull-save [-h] [--cache-dir CACHE_DIR] [--verbose]
                                  NAME[:TAG|@DIGEST]

Example::

    $ python anyci/docker.py load-pull-save hello-world:latest
    [anyci:docker.py] Loading cached image from /home/jcfr/docker/hello-world-latest.tar
    [anyci:docker.py]   -> cached image not found
    [anyci:docker.py] Pulling image: hello-world:latest
    [anyci:docker.py]   -> done
    [anyci:docker.py] Reading image ID from current image
    [anyci:docker.py]   -> image ID: sha256:c54a2cc56cbb2f04003c1cd4507e118af7c0d340fe7e2720f70976c4b75237dc
    [anyci:docker.py] Caching image
    [anyci:docker.py]   -> image cached: /home/jcfr/docker/hello-world-latest.tar
    [anyci:docker.py] Saving image ID into /home/jcfr/docker/hello-world-latest.image_id
    [anyci:docker.py]   -> done

Notes:

- Image is saved into the cache only if needed. In addition to the image
  archive (e.g `image-name.tar`), a file containing the image ID is also
  saved into the cache directory (e.g `image-name.image_id`). This allows
  to quickly read back the image ID of the cached image and determine if
  the current image should be saved into the cache.

"""  # noqa: E501

import argparse
import os
import re
import subprocess
import sys


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[anyci:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def get_valid_filename(s):
    """
    Returns the given string converted to a string that can be used for a clean
    filename. Specifically, leading and trailing spaces are removed; other
    spaces are converted to underscores; slashes and colons are converted to
    dashes; and anything that is not a unicode alphanumeric, dash, underscore,
    or dot, is removed.
    >>> get_valid_filename("john's portrait in 2004.jpg")
    'johns_portrait_in_2004.jpg'
    >>> get_valid_filename("library/hello-world:latest")
    'library-hello-world-latest'

    Copied from https://github.com/django/django/blob/20be1918e77414837178d6bf1657068c8306d50c/django/utils/encoding.py
    Distributed under BSD-3 License
    """  # noqa: E501
    s = s.strip().replace(' ', '_').replace('/', '-').replace(':', '-')
    return re.sub(r'(?u)[^-\w.]', '', s)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    subparsers = parser.add_subparsers(help='sub-command help')

    # create the parser for the "load-pull-save" command
    parser_pull = subparsers.add_parser("load-pull-save",
                                        help="load-pull-save help")
    parser_pull.add_argument(
        "image", type=str, metavar="NAME[:TAG|@DIGEST]",
        help="Load an image from local cache, pull and save back"
    )

    parser_pull.add_argument(
        "--cache-dir", type=str, metavar="CACHE_DIR", default="~/docker",
        help="Image cache directory (default: ~/docker)"
    )

    parser_pull.add_argument(
        "--verbose", action="store_true",
        help="Display pulling progress"
    )

    args = parser.parse_args()

    if hasattr(args, 'image'):

        # If needed, create cache directory
        cache_dir = os.path.expanduser(args.cache_dir)
        if not os.path.exists(cache_dir):
            os.mkdir(cache_dir)

        # Convert image to valid filename
        filename = os.path.join(cache_dir, get_valid_filename(args.image))
        image_filename = filename + '.tar'
        image_id_filename = filename + '.image_id'

        # If it exists, load cache image
        cached_image_id = ""
        _log("Loading cached image", "from", image_filename)
        if os.path.exists(image_filename):
            cmd = ["docker", "load", "-i", image_filename]
            subprocess.check_output(cmd)
            _log("  ->", "done")

            # Read image id
            if os.path.exists(image_id_filename):
                _log("Reading cached image ID", "from", image_id_filename)
                with open(image_id_filename) as content:
                    cached_image_id = content.readline()
                _log("  ->", "cached image ID:", cached_image_id)

        else:
            _log("  ->", "cached image not found")

        # Pull latest image if any
        _log("Pulling image:", args.image)
        cmd = ["docker", "pull", args.image]
        (subprocess.check_call
         if args.verbose else subprocess.check_output)(cmd)
        _log("  ->", "done")

        # Get ID of current image
        _log("Reading image ID from current image")
        cmd = ["docker", "inspect", "--format='{{.Config.Image}}'", args.image]
        output = subprocess.check_output(cmd).decode("utf-8")
        current_image_id = output.strip()
        _log("  ->", "image ID:", current_image_id)

        # Cache image only if updated
        if cached_image_id != current_image_id:
            _log("Caching image")
            cmd = ["docker", "save", "-o", image_filename, args.image]
            subprocess.check_output(cmd)
            _log("  ->", "image cached:", image_filename)

            _log("Saving image ID into", image_id_filename)
            with open(image_id_filename, "w") as content:
                content.write(current_image_id)
            _log("  ->", "done")
        else:
            _log("Caching image")
            _log("  ->", "Skipped because pulled image did not change")
    else:
        parser.print_usage()


if __name__ == '__main__':
    main()




"""
Usage::

    import install_cmake
    install_cmake.install()

"""

import os
import subprocess
import sys

from subprocess import CalledProcessError, check_output

DEFAULT_CMAKE_VERSION = "3.5.0"


def _log(*args):
    script_name = os.path.basename(__file__)
    print("[circle:%s] " % script_name + " ".join(args))
    sys.stdout.flush()


def install(cmake_version=DEFAULT_CMAKE_VERSION):
    """Download and install CMake into ``/usr/local``."""

    cmake_directory = "/usr/local"

    cmake_exe = os.path.join(cmake_directory, 'bin/cmake')

    if os.path.exists(cmake_exe):
        output = check_output([cmake_exe, '--version']).decode("utf-8")
        if output.strip() == cmake_version:
            _log("Skipping download: Found %s (v%s)" % (
                cmake_exe, cmake_version))
            return

    _log("Looking for cmake", cmake_version, "in PATH")
    try:
        output = check_output(
            "cmake --version", shell=True).decode("utf-8")
        current_cmake_version = output.splitlines()[0]
        if cmake_version in current_cmake_version:
            _log("  ->", "found %s:" % current_cmake_version,
                 "skipping download: version matches expected one")
            return
        else:
            _log("  ->", "found %s:" % current_cmake_version,
                 "not the expected version")
    except (OSError, CalledProcessError):
        _log("  ->", "not found")
        pass

    name = "cmake-{}-Linux-x86_64".format(cmake_version)

    cmake_package = "{}.tar.gz".format(name)

    _log("Downloading", cmake_package)

    if not os.path.exists(cmake_package):
        cmake_version_major = cmake_version.split(".")[0]
        cmake_version_minor = cmake_version.split(".")[1]

        check_output([
            "wget", "--no-check-certificate", "--progress=dot",
            "https://cmake.org/files/v{}.{}/{}".format(
                cmake_version_major, cmake_version_minor, cmake_package)
        ], stderr=subprocess.STDOUT)
        _log("  ->", "done")
    else:
        _log("  ->", "skipping download: found", cmake_package)

    _log("Extracting", cmake_package)
    check_output(["tar", "xzf", name + ".tar.gz"])
    _log("  ->", "done")

    _log("Installing", name, "into", cmake_directory)
    check_output([
        "sudo", "rsync", "-avz", name + "/", cmake_directory
    ])
    _log("  ->", "done")


if __name__ == '__main__':
    install(sys.argv[1] if len(sys.argv) > 1 else DEFAULT_CMAKE_VERSION)



# -*- coding: utf-8 -*-

"""
scikit-ci-addons is a command line tool and a set of scripts useful to install
prerequisites for building Python extension on CI services.
"""

import os
import shutil
import sys

from subprocess import CalledProcessError, check_call

from ._version import get_versions

__author__ = 'The scikit-build team'
__email__ = 'scikit-build@googlegroups.com'
__version__ = get_versions()['version']
del get_versions

DIR_NAMES = ['anyci', 'appveyor', 'circle', 'travis']


class SKAddonsError(RuntimeError):
    """Exception raised when a user error occurs.
    """
    pass


def addons():
    """Return all available add-ons."""

    addons = []

    for dirname, dirnames, filenames in os.walk(home()):

        for v in list(dirnames):
            dirnames.remove(v)
        dirnames += DIR_NAMES

        if dirname == home():
            continue

        for filename in filenames:
            if filename in ['__init__.py'] or filename.endswith(".pyc"):
                continue
            addon_path = os.path.join(dirname, filename)
            addons.append(os.path.relpath(addon_path, home()))

    return addons


def home():
    """Return directory where all add-ons can be found."""
    return os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))


def path(addon_name):
    """Return path of ``addon_name``.

    Supported values for ``addon_name`` are listed below:
    - relative path with or without extension (e.g ``appveyor/patch_vs2008.py``
      or ``appveyor/patch_vs2008.py``)
    - full path (e.g ``/path/to/appveyor/patch_vs2008.py``
    - script name with or without extension (e.g ``patch_vs2008.py``
      or ``patch_vs2008``). If there are multiple matching scripts, a
      ``SKAddonsError`` exception is raised.
    """
    def _path(_addon_name):
        _addon_path = os.path.join(dir_name, home(), _addon_name)
        if (not os.path.exists(_addon_path)
                and not _addon_path.endswith(".py")):
            _addon_path += '.py'
        return _addon_path if os.path.exists(_addon_path) else ""

    candidates = []
    for dir_name in DIR_NAMES + [""]:
        addon_path = _path(os.path.join(dir_name, addon_name))
        if addon_path and addon_path not in candidates:
            candidates.append(addon_path)
    if len(candidates) > 1:
        raise SKAddonsError(
            "Failed to return a single path because it found %d matching "
            "paths. You must select one of these:\n  %s" % (
                len(candidates), "\n  ".join(candidates)))
    elif len(candidates) == 1:
        return candidates[0]
    else:
        raise SKAddonsError("Could not find addon: %s" % addon_name)


def install(dst_path, force=False):
    """Copy add-ons into ``dst_path``.

    By default, existing add-ons are *NOT* overwritten. Specifying ``force``
    allow to overwrite them.
    """
    dst_path = os.path.normpath(os.path.abspath(dst_path))
    if dst_path == os.path.normpath(home()):
        raise SKAddonsError(
            "skipping install: target directory already contains add-ons")
    for addon in addons():
        dst_addon_path = os.path.join(dst_path, addon)
        dst_addon_dir = os.path.split(dst_addon_path)[0]
        if not os.path.exists(dst_addon_dir):
            os.makedirs(dst_addon_dir)
        src_addon_path = os.path.join(home(), addon)
        extra = ""
        do_copy = True
        if os.path.exists(dst_addon_path):
            extra = " (skipped)"
            do_copy = False
            if force:
                extra = " (overwritten)"
                do_copy = True
        if do_copy:
            shutil.copy(src_addon_path, dst_addon_path)
        print(dst_addon_path + extra)


def execute(addon_name, arguments=[]):
    """Execute ``addon_name`` with ``arguments``.

    Executable add-ons are python script.
    """
    cmd = [sys.executable, path(addon_name)] + arguments
    try:
        check_call(cmd)
    except CalledProcessError as error:
        sys.exit(error.returncode)



# -*- coding: utf-8 -*-

"""
Ci Addons command line tool
"""

import argparse
import ci_addons
import os
import sys


def main():
    """The main entry point to ``ci_addons``.

    This is installed as the script entry point.
    """

    version_str = ("This is scikit-ci-addons version %s, imported from %s\n" %
                   (ci_addons.__version__, os.path.abspath(ci_addons.__file__)))

    parser = argparse.ArgumentParser(description=ci_addons.__doc__)
    parser.add_argument(
        'addon', metavar='ADDON', type=str, nargs='?',
        help='name of add-on to execute'
    )
    parser.add_argument(
        'arguments', metavar='ARG', type=str, nargs='*',
        help='add-on arguments'
    )
    parser.add_argument(
        "--home", action="store_true",
        help="display directory where all add-ons can be found"
    )
    parser.add_argument(
        "--list", action="store_true",
        help="list all available add-ons"
    )
    parser.add_argument(
        "--path", type=str,
        help="display add-on path"
    )
    parser.add_argument(
        "--install", type=str, metavar="DIR",
        help="install add-ons in the selected directory"
    )
    parser.add_argument(
        "--version", action="version",
        version=version_str,
        help="display scikit-ci-addons version and import information"
    )

    # If an add-on is selected, let's extract its arguments now. This will
    # prevent ci_addons parser from complaining about unknown parameters.
    addon_arguments = []
    if len(sys.argv) > 1:
        try:
            ci_addons.path(sys.argv[1])
            addon_arguments = sys.argv[2:]
            if len(addon_arguments) > 0 and addon_arguments[0] == '--':
                addon_arguments.pop(0)
            sys.argv = sys.argv[:2]
        except ci_addons.SKAddonsError:
            pass

    args = parser.parse_args()
    args.arguments = addon_arguments

    try:

        if args.home:  # pragma: no cover
            print(ci_addons.home())
            exit()

        if args.list:
            previous_collection = ""
            for addon in ci_addons.addons():
                current_collection = addon.split(os.path.sep)[0]
                if previous_collection != current_collection:
                    print("")
                print(addon)
                previous_collection = current_collection
            exit()

        if args.path is not None:  # pragma: no cover
            print(ci_addons.path(args.path))
            exit()

        if args.install is not None:  # pragma: no cover
            ci_addons.install(args.install)
            exit()

        if all([not getattr(args, arg)
                for arg in ['addon', 'home', 'install', 'list', 'path']]):
            parser.print_usage()
            exit()

        ci_addons.execute(args.addon, args.arguments)

    except ci_addons.SKAddonsError as error:
        exit(error)


if __name__ == '__main__':  # pragma: no cover
    main()
